# ZPyIrc
#
# Copyright (c) 2003 by Sylvain Hellegouarch.
#
# This software is released under the terms of the GNU General
# Public License version 2, or (at your option) any later
# version. It is distributed WITHOUT ANY WARRANTY. See the GNU
# General Public License for more details.
#

#Zope specific import
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from ImageFile import ImageFile
from Acquisition import Implicit
from Globals import Persistent
from OFS.SimpleItem import Item
from Products.PythonScripts.Utility import allow_module
allow_module('pyirc')
import zLOG

#Python specific import
import string
from time import strftime,gmtime,time
from sys import platform

#Let's import the core module
import pyirc

#Manage connexions
_pyirc_objects = {}

if platform == 'win32':
    _default_separator = "\r\n"
elif platform == 'linux1':
    _default_separator = "\n"
else:
    _default_separator = "\n"

def manage_addZPyIrcConnexion(self,id,server,port,channel,nickname,username,realname,password='',host='localhost'):
    """Add ZPyIrcConnexion"""
    self._setObject(id, ZPyIrcConnexion(id,id,server,port,channel,nickname,username,realname))

# Is it useful to inherit from Persistent here ?
class ZPyIrcChannel:
    def __init__(self,name):
        self.name = name
        self.nicks = {}
        self.text = "ZPyIrc is connecting..." + _default_separator
        self.active = 0
        
    def get_name(self):
        return self.name

    def set_active(self,value):
        self.active = value

    def is_active(self):
        return self.active
    
    def add_text(self,text):
        self.text = self.text + text + _default_separator

    def get_text(self):
        return self.text

    def clear_text(self):
        self.text = "ZPyIrc is connecting..." + _default_separator

    def set_nicks(self,nicks):
        for nick in nicks:
            if nick[0] == '@':
                self.nicks[nick[1:]] = '@'
            elif nick[0] == '+':
                self.nicks[nick[1:]] = '+'
            else:
                self.nicks[nick] = ''
        
    def add_nick(self,nick):
        self.nicks[nick] = ''

    def get_nicks(self):
        nicks = []
        for nick,mode in self.nicks.items():
            if mode in ['@','@+']:
                nicks.append(str('@'+nick))
            elif mode == '+':
                nicks.append(str('+'+nick))
            else:
                nicks.append(nick)
        return nicks

    def del_nick(self,nick):
        if nick in self.nicks.keys():
            del self.nicks[nick]

    def get_mode_for(self,nick):
        if nick in self.nicks.keys():
            return self.nicks[nick]
        return ''

    def set_mode_for(self,nick,mode):
        self.nicks[nick] = mode

class ZPyIrcConnexion(Implicit, Persistent, Item):
    meta_type = "ZPyIrcConnexion"
    index_html = PageTemplateFile('ui/ZPyIrcConnexionInterface_view.pt', globals())
    view_main_area = PageTemplateFile('ui/ZPyIrcConnexionInterface_main_area.pt', globals())
    view_head = PageTemplateFile('ui/ZPyIrcConnexionInterface_head.pt', globals())
    bye = PageTemplateFile('ui/ZPyIrcConnexionInterface_bye.pt', globals())
    
    def __init__(self,id,title,server,port,channel,nickname,username,realname,password='',host='localhost'):
        """ZPyIrcConnexion class"""
        self.id = id
        self.title = title
        self.server = server
        self.nickname = nickname
        self.username = username
        self.realname = realname
        self.password = password
        self.host = host
        self.away = 0
        self.channels = {}
        self.help_asked = 0
        self.current_active_channel = ZPyIrcChannel(channel)
        self.channels[channel] = self.current_active_channel
        self.idle_duration = time()
        self.is_connected = 0
        connexion = pyirc.IRCServerConnexion(server,port,nickname,username,realname,password,host)
        try:
            connexion.connect()
            connexion.join(channel)
            self.current_active_channel.set_active(1)
            self.is_connected = 1
        except:
            raise RuntimeError , str("Can't connect to " + server + " on port " + str(port))
        _pyirc_objects[self.id] = connexion

    def restart_connexion(self,server,port,channel):
        """Restart a connexion"""
        self.away = 0
        self.channels = {}
        self.help_asked = 0
        self.idle_duration = time()
        self.current_active_channel = ZPyIrcChannel(channel)
        self.channels[channel] = self.current_active_channel
        self.idle_duration = time()
        connexion = pyirc.IRCServerConnexion(server,port,self.nickname,self.username,self.realname,self.password,self.host)
        try:
            connexion.connect()
            connexion.join(channel)
            self.current_active_channel.set_active(1)
            self.is_connected = 1
        except:
            raise RuntimeError , str("Can't connect to " + server + " on port " + str(port))
        _pyirc_objects[self.id] = connexion
        
    def manage_beforeDelete(self,item,container):
        """Tell the server you're leaving and close the socket connexion nicely"""
        if self.id in _pyirc_objects.keys():
            _pyirc_objects[self.id].quit()
            _pyirc_objects[self.id].disconnect()
            del _pyirc_objects[self.id]
            self.is_connected = 0
            self._p_changed = 1

    def is_alive(self):
        """Return true if the connexion is still up"""
        return self.is_connected
    
    def get_channels_name(self):
        """Get a list of name of all channels"""
        return self.channels.keys()

    def get_current_active_channel_name(self):
        """Get the name of the current active channel"""
        return self.current_active_channel.get_name()

    def set_channel_as_active(self,REQUEST=None):
        """Set 'channel' as active and de-active the previous one"""
        self.current_active_channel.set_active(0)
        self.current_active_channel = self.channels[self.REQUEST['activate_channel_list']]
        self.current_active_channel.set_active(1)
        self._p_changed = 1
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('view_main_area')

    def _set_channel_as_active_internal(self,channel):
        """Set 'channel' as active and de-active the previous one"""
        self.current_active_channel.set_active(0)
        self.current_active_channel = self.channels[channel]
        self.current_active_channel.set_active(1)
        self._p_changed = 1
        
    def clear_current_active_channel_text(self):
        """Clear the text of the active channel"""
        self.current_active_channel.clear_text()
        self._p_changed = 1

    def get_current_active_channel_text(self):
        """Get the text of the active channel"""
        if not self.is_connected:
            return "You are not connected to the server" + _default_separator
        return self.current_active_channel.get_text()

    def get_current_active_channel_nicknames(self):
        """Get the nicknames list of the active channel"""
        if not self.is_connected:
            return []
        return self.current_active_channel.get_nicks()

    def get_current_active_channel_id(self):
        """Get the id of the active channel"""
        if not self.is_connected:
            return 0
        return self.channels.keys()[self.current_active_channel.get_name()].index()
    
    def _send_common_message(self,message):
        """Send a simple message (not a command) to the active channel"""
        text = "<" + self.nickname + "> " + message
        self.current_active_channel.add_text(text)
        _pyirc_objects[self.id].privmsg(self.current_active_channel.get_name(),message)

    def _send_join(self,channel):
        """Join a new channel"""
        if self.channels.has_key(channel):
            return
        self.channels[channel] = ZPyIrcChannel(channel)
        _pyirc_objects[self.id].join(channel)
        self._set_channel_as_active_internal(channel)
        self._p_changed = 1

    def _send_part(self,channel):
        """Leave a channel or a user query"""
        del self.channels[channel]
        if channel[0] in ['#','&','!']:
            _pyirc_objects[self.id].part(channel)
        self.set_channel_as_active(self.channels.keys()[0])
        self._p_changed = 1

    def _send_nick(self,nick):
        """Change your nickname"""
        _pyirc_objects[self.id].nick(nick)
        
    def _send_query(self,who,message):
        """Send a query to user"""
        if not self.channels.has_key(who):
            self.channels[who] = ZPyIrcChannel(who)
        text = "<" + self.nickname + "> " + message
        self.channels[who].add_text(text)
        _pyirc_objects[self.id].privmsg(who,message)
        self._set_channel_as_active_internal(who)
        self._p_changed = 1
        
    def _send_ctcp_action(self,message):
        """Send a /me message"""
        text = "* " + self.nickname + message
        self.current_active_channel.add_text(text)
        _pyirc_objects[self.id].ctcp_action(self.current_active_channel.get_name(),message)

    def _send_away(self,message=""):
        """Tell the server you're away"""
        _pyirc_objects[self.id].away(message)

    def _send_back(self):
        """Tell the server you're back"""
        _pyirc_objects[self.id].away()

    def _send_topic(self,topic=""):
        """Set/Ask the topic of the current channel"""
        _pyirc_objects[self.id].topic(self.current_active_channel.get_name(),topic)

    def _send_userhost(self,user):
        """Ask user host information"""
        _pyirc_objects[self.id].userhost(user)

    def _send_kick(self,user):
        """Kick an user from the current channel"""
        _pyirc_objects[self.id].kick(user)

    def _send_voice(self,user):
        """Voice an user"""
        _pyirc_objects[self.id].mode_channel(self.current_active_channel.get_name(),"+v",user)
    
    def _send_unvoice(self,user):
        """Unvoice an user"""
        _pyirc_objects[self.id].mode_channel(self.current_active_channel.get_name(),"-v",user)
    
    def _send_op(self,user):
        """Op an user"""
        _pyirc_objects[self.id].mode_channel(self.current_active_channel.get_name(),"++",user)
    
    def _send_deop(self,user):
        """Deop an user"""
        _pyirc_objects[self.id].mode_channel(self.current_active_channel.get_name(),"-o",user)

    def _send_invite(self,user,channel):
        """Invite user to channel"""
        _pyirc_objects[self.id].invite(user,channel)
        
    def say_something(self,REQUEST=None):
        """Say something to the server"""
        self.idle_duration = time()
        r = self.REQUEST
        message = ""
        if r.has_key('say_whatever'):
            message = r['say_whatever']
        else:
            return self.REQUEST.RESPONSE.redirect('view_main_area')
        
        if message[0] != "/":
            if message != "":
                self._send_common_message(message)
        else:
            data = string.split(message,' ',1)
            if data[0] == "/join":
                if len(data) == 2:
                    self._send_join(data[1])
            elif data[0] == "/part":
                if len(data) == 1:
                    self._send_part(self.current_active_channel.get_name())
                else:
                    self._send_part(data[1])
            elif data[0] == "/nick":
                if len(data) == 2:
                    self._send_nick(data[1])
            elif data[0] == "/msg":
                if len(data) == 2:
                    d = string.split(data[1],' ',1)
                    if len(d) == 2:
                        self._send_query(d[0],d[1])
            elif data[0] == "/me":
                if len(data) == 2:
                    self._send_ctcp_action(data[1])
            elif data[0] == "/away":
                if self.away == 0:
                    self.away = 1
                    if len(data) == 1:
                        self._send_away()
                    else:
                        self._send_away(data[1])
                else:
                    self.away = 0
                    self._send_back()
            elif data[0] == "/topic":
                if len(data) == 2:
                    self._send_topic(data[1])
                else:
                    self._send_topic()
            elif data[0] == "/userhost":
                if len(data) == 2:
                    self._send_userhost(data[1])
            elif data[0] == "/kick":
                if len(data) == 2:
                    self._send_kick(data[1])
            elif data[0] == "/voice":
                if len(data) == 2:
                    self._send_voice(data[1])
            elif data[0] == "/unvoice":
                if len(data) == 2:
                    self._send_unvoice(data[1])
            elif data[0] == "/op":
                if len(data) == 2:
                    self._send_op(data[1])
            elif data[0] == "/deop":
                if len(data) == 2:
                    self._send_deop(data[1])
            elif data[0] == "/invite":
                if len(data) == 2:
                    tmp = string.split(data[1]," ")
                    if len(tmp) == 2:
                        self._send_invite(tmp[0],tmp[1])
                    else:
                        self._send_invite(tmp[0],self.current_active_channel.get_name())
            elif data[0] == "/help":
                self.help_asked = 1
            elif data[0] == "/quit":
                if len(data) == 1:
                    if self.id in _pyirc_objects.keys():
                        _pyirc_objects[self.id].quit()
                else:
                    if self.id in _pyirc_objects.keys():
                        _pyirc_objects[self.id].quit(data[1])
                _pyirc_objects[self.id].disconnect()
                del _pyirc_objects[self.id]
                self.is_connected = 0
                self._p_changed = 1
                return self.REQUEST.RESPONSE.redirect('bye')
        self._p_changed = 1
        return self.REQUEST.RESPONSE.redirect('view_main_area')
                        
    def get_next_message(self):
        """Get the next message queuing"""
        _pyirc_objects[self.id].process()
        next_msg = []
        text = ""
        while _pyirc_objects[self.id].has_more_messages():
            msg = _pyirc_objects[self.id].get_next_message()
            if msg[0] == "privmsg":
                if msg[4][0] not in ['#','&','!']:
                    if not self.channels.has_key(msg[3]):
                        self.channels[msg[3]] = ZPyIrcChannel(msg[3])
                        self._set_channel_as_active_internal(msg[3])
                        next_msg.append({"query":msg[3]})
                    channel = msg[3]
                else:
                    channel = msg[4]
                text = str("<" + msg[3] + "> " + msg[5][:-2])
                if self.current_active_channel.get_name() == channel:
                    next_msg.append({"privmsg":text})
                self.channels[channel].add_text(text)
            elif msg[0] == "ping":
                _pyirc_objects[self.id].pong(self.server)
            elif msg[0] == "ctcp":
                if msg[5] == "ACTION":
                    text = str("* " + msg[3] + " " + msg[6])
                    channel = msg[4]
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"ctcp":text})
                    self.channels[channel].add_text(text)
                elif msg[5] == "VERSION":
                    _pyirc_objects[self.id].ctcp_version(msg[3],"ZPyIrc","0.1",platform)
                elif msg[5] == "TIME":
                    _pyirc_objects[self.id].ctcp_time(msg[3],strftime("%a, %d %b %Y %H:%M:%S", gmtime()))
                elif string.find(msg[5],"PING",0) == 0:
                    _pyirc_objects[self.id].ctcp_ping(msg[3],msg[6])
            elif msg[0] == "nick":
                if msg[3] == self.nickname:
                    text = str("* You are now known as " + msg[4][:-2])
                    next_msg.append({"nick":text})
                    for key,channel in self.channels.items():
                        channel.add_text(text)
                    next_msg.append({"update_nick":str(self.nickname + " " + msg[4][:-2])})
                    self.nickname = msg[3]
                    self._p_changed = 1
                else:
                    text = str("* " + msg[3] + " is now known as " + msg[4][:-2])
                    next_msg.append({"nick":text})
                    for key,channel in self.channels.items():
                        channel.add_text(text)
                    next_msg.append({"update_nick":str(msg[3] + " " + msg[4][:-2])})
            elif msg[0] == "join":
                if msg[3] == self.nickname:
                    text = str("Now talking on " + msg[4][:-2])
                    channel = msg[4][:-2]
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"join":text})
                    self.channels[channel].add_text(text)
                else:
                    text = str("* " + msg[3] + "(" + msg[2] + ")" + " has joined " + msg[4][:-2])
                    next_msg.append({"join":text})
                    channel = msg[4][:-2]
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"add_nick":str(msg[3])})
                    self.channels[channel].add_nick(msg[3])
                    self.channels[channel].add_text(text)
            elif msg[0] == "part":
                if msg[3] != self.nickname:
                    text = str("* " + msg[3] + " has left (" + msg[5][:-2] + ")")
                    channel = msg[4]
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"part":text})
                    self.channels[channel].del_nick(msg[3])
                    self.channels[channel].add_text(text)
                    next_msg.append({"del_nick":str(msg[3])})
            elif msg[0] == "mode":
                if self.nickname != msg[2]:
                    channel = msg[4]
                    mode = self.channels[channel].get_mode_for(msg[6])
                    if msg[5] == "+o":
                        text = str("* " + msg[3] + " gives channel operator status to " + msg[6])
                        if mode == '':
                            self.channels[channel].set_mode_for(msg[6],'@')
                        elif mode == '+':
                            self.channels[channel].set_mode_for(msg[6],'@+')
                    elif msg[5] == "-o":
                        text = str("* " + msg[3] + " removes channel operator status from " + msg[6])
                        if mode == '@':
                            self.channels[channel].set_mode_for(msg[6],'')
                        elif mode == '@+':
                            self.channels[channel].set_mode_for(msg[6],'+')
                    elif msg[5] == "+v":
                        text = str("* " + msg[3] + " gives voice to " + msg[6])
                        if mode == '':
                            self.channels[channel].set_mode_for(msg[6],'+')
                        elif mode == '@':
                            self.channels[channel].set_mode_for(msg[6],'@+')
                    elif msg[5] == "-v":
                        text = str("* " + msg[3] + " removes voice from " + msg[6])
                        if mode == '+':
                            self.channels[channel].set_mode_for(msg[6],'')
                        elif mode == '@+':
                            self.channels[channel].set_mode_for(msg[6],'@')
                    self.channels[channel].add_text(text)
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"mode":text})
                        mode = self.channels[channel].get_mode_for(msg[6])
                        if mode == "@+":
                            mode = "@"
                        next_msg.append({"update_mode":str(mode + " " + msg[6])})
            elif msg[0] == "invite":
                text = "* You have been invited to " + msg[5][:-2] + " by " + msg[3] + " (" + msg[1] + ")"
                self.current_active_channel.add_text(text)
                next_msg.append({'invite':text})
            elif msg[0] in ["markedaway","unmarkedaway"]:
                text = msg[3][:-2]
                self.current_active_channel.add_text(text)
                next_msg.append({'away':text})
            elif msg[0] == "isaway":
                text = "* [" + msg[3] + "] is away : " + msg[4][:-2]
                if self.channels.has_key(msg[3]):
                    self.channels[msg[3]].add_text(text)
                else:
                    self.current_active_channel.add_text(text)
                next_msg.append({'isaway':text})
            elif msg[0] == "quit":
                if msg[3] != self.nickname:
                    text = "* " + msg[3] + " has quit (" + msg[4][:-2] + ")"
                    for channel in self.channels.values():
                        if msg[3] in channel.get_nicks():
                            channel.add_text(text)
                            channel.del_nick(msg[3])
                            if channel.get_name() == self.current_active_channel.get_name():
                                next_msg.append({"quit":text})
                                next_msg.append({"del_nick":str(msg[3])})
            elif msg[0] == "topic":
                channel = msg[4]
                text1 = "* Topic is " + msg[5][:-2]
                text2 = "* Set by " + msg[3]
                self.channels[channel].add_text(text1)
                self.channels[channel].add_text(text2)
                if self.current_active_channel.get_name() == channel:
                    next_msg.append({"topic":text1})
                    next_msg.append({"topic_author":text2})
            elif msg[0] == "notopic":
                text = "* No topic is set for " + msg[4]
                channel = msg[4]
                self.channels[channel].add_text(text)
                if self.current_active_channel.get_name() == channel:
                    next_msg.append({"notopic":text})
            elif msg[0] == "datanames":
                if len(msg[3]) != 0:
                    text = str("* Users on " + msg[2] + " " + string.join(msg[3],' '))
                    next_msg.append({"names":text})
                    channel = msg[2]
                    if self.current_active_channel.get_name() == channel:
                        next_msg.append({"nick_list":str(string.join(msg[3],' '))})
                    self.channels[channel].add_text(text)
                    self.channels[channel].set_nicks(msg[3])
            elif msg[0] == "userhost":
                data = re.compile(r"^([\w]*)(\*?)=([-|\+])(\S*)").search(msg[3])
                if data != None: 
                    text = "* " + data.group(1) + " [" + data.group(4) + "]"
                    if data.group(3) == '-':
                        text = text + " " + "[away]"
                    if data.group(2) == "*":
                        text = text + " " + "[operator]"
                    self.current_active_channel.add_text(text)
                    next_msg.append({"userhost":text})
            elif msg[0] == "error":
                text = "! " + msg[3] + " :" + msg[4][:-2]
                self.current_active_channel.add_text(text)
                next_msg.append({'error':text})
        if self.help_asked == 1:
            self.help_asked = 0
            text = "Here are the supported commands :"+_default_separator+"/quit"+_default_separator+"/join channel"+_default_separator+"/part [channel/query]"+_default_separator+"/nick nickname"+_default_separator+"/me message"+_default_separator+"/msg user message"+_default_separator+"/away [message]"+_default_separator+"/topic [topic]"+_default_separator+"/userhost user"+_default_separator+"/kick user"+_default_separator+"/voice user"+_default_separator+"/unvoice user"+_default_separator+"/op user"+_default_separator+"/deop user"+_default_separator+"/invite user channel"+_default_separator
            self.current_active_channel.add_text(text)
            next_msg.append({"help":text})
        if len(next_msg) == 0:
            next_msg = ""
            if (time() - self.idle_duration) > self.aq_parent.timeout:
                self.aq_parent.delete_client(self.id)
        else:
            if len(next_msg) != 0:
                next_msg = str(next_msg)
            else:
                next_msg = ""
            self.idle_duration = time()
            self._p_changed = 1
        return next_msg
            
    
InitializeClass(ZPyIrcConnexion)
